package cn.com.qmth.dp.examcloud.oe.modules.export_exam_student_score;

import cn.com.qmth.dp.examcloud.oe.modules.export_exam_student_score.vo.ExamQuestionVO;
import cn.com.qmth.dp.examcloud.oe.modules.export_exam_student_score.vo.ExamRecordQuestionVO;
import cn.com.qmth.dp.examcloud.oe.modules.export_exam_student_score.vo.ExamStudentVO;
import cn.com.qmth.dp.examcloud.oe.modules.export_exam_student_score.vo.ScoreVO;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelWriter;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.safety.Safelist;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 导出 - 考生的考试记录中小题的作答内容和得分明细
 */
@Component
public class ExportExamStudentScore {

    private static Logger log = LoggerFactory.getLogger(ExportExamStudentScore.class);

    private static final Long MARKING_TYPE_KEY = 24L;// 阅卷方式KEY

    private static final boolean needSubjective = true;// 是否包含主观题

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Autowired
    private MongoTemplate mongoTemplate;

    public void start(Long... examIds) {
        for (Long examId : examIds) {
            this.export(examId);
        }
    }

    private void export(Long examId) {
        log.info("===> export starting, examId = " + examId);

        // 获取考试下待导的课程
        List<Long> examCourseIds = this.queryExamCourseIds(examId);
        log.info("---> examCourseIds size is " + examCourseIds.size());

        // 获取考试的阅卷方式
        String markingType = this.queryExamMarkingType(examId, MARKING_TYPE_KEY);
        log.info("---> markingType is " + markingType);

        String examName = this.queryExamName(examId);

        Map<Long, String> orgNameMaps = new HashMap<>(), courseNameMaps = new HashMap<>();

        for (Long courseId : examCourseIds) {
            // 按考试的每个课程依次导出
            this.export(examId, examName, courseId, markingType, orgNameMaps, courseNameMaps);
        }

        log.info("===> export finished, examId = " + examId + "\n");
    }

    private void export(Long examId, String examName, Long courseId, String markingType, Map<Long, String> orgNameMaps, Map<Long, String> courseNameMaps) {
        List<ExamStudentVO> examStudents = this.queryExamStudents(examId, courseId);
        log.info("---> examId = {}, courseId = {}, examStudentSize = {}", examId, courseId, examStudents.size());

        boolean hasDynamicExcelHeaderSetting = false;
        List<String> dynamicExcelHeaders = new ArrayList<>();// Excel动态列
        List<String> defaultDynamicColumnValues = new ArrayList<>();// Excel动态列默认值
        String paperStructKey = "";// 试卷结构关键值

        int index = 0, total = examStudents.size();
        for (ExamStudentVO examStudent : examStudents) {
            index++;

            if (!orgNameMaps.containsKey(examStudent.getOrgId())) {
                String orgName = this.queryOrgName(examStudent.getOrgId());
                orgNameMaps.put(examStudent.getOrgId(), orgName);
            }

            if (!courseNameMaps.containsKey(examStudent.getCourseId())) {
                String courseName = this.queryCourseName(examStudent.getCourseId());
                courseNameMaps.put(examStudent.getCourseId(), courseName);
            }

            // 设置学习中心名称、课程名称
            examStudent.setOrgName(orgNameMaps.get(examStudent.getOrgId()));
            examStudent.setCourseName(courseNameMaps.get(examStudent.getCourseId()));

            // 成绩默认值
            examStudent.setObjectiveScore(0d);
            examStudent.setSubjectiveScore(0d);
            examStudent.setTotalScore(0d);

            if (!examStudent.getFinished()) {
                // 跳过缺考的情况
                log.warn("---> {}-{} examStudentId={}, finished=false", total, index, examStudent.getExamStudentId());
                continue;
            }

            // 获取考试记录
            List<Map<String, Object>> examRecordDataList = this.queryExamRecordDataList(examId, courseId, examStudent.getExamStudentId());
            if (CollectionUtils.isEmpty(examRecordDataList)) {
                // 跳过暂无有效考试记录的情况
                log.warn("---> {}-{} examStudentId={}, examRecordData is empty", total, index, examStudent.getExamStudentId());
                continue;
            }

            Long okExamRecordDataId;
            if (examRecordDataList.size() == 1) {
                // 只有一条考试记录情况
                okExamRecordDataId = (Long) examRecordDataList.get(0).get("id");

                // 获取考试记录对应的成绩
                ScoreVO okScore = this.queryExamScore(okExamRecordDataId);
                examStudent.setObjectiveScore(okScore.getObjectiveScore());
                examStudent.setSubjectiveScore(okScore.getSubjectiveScore());
                examStudent.setTotalScore(okScore.getTotalScore());

                // System.out.println("base_paper_id = " + examRecordDataList.get(0).get("base_paper_id"));
                // db.getCollection('paper').find({_id:ObjectId("5ed67e2bc68d4c520320b5b7")})
            } else {
                // 多条考试记录情况
                List<ScoreVO> scores = new ArrayList<>();
                for (Map<String, Object> examRecordData : examRecordDataList) {
                    Long examRecordDataId = (Long) examRecordData.get("id");
                    // 分别获取考试记录对应的成绩
                    ScoreVO score = this.queryExamScore(examRecordDataId);
                    scores.add(score);
                }

                ScoreVO okScore;
                if (markingType.equals("ALL") || markingType.equals("OBJECT_SCORE_MAX")) {
                    // 全部评阅规则 或 客观分最高规则：取总分最高
                    okScore = scores.stream()
                            .sorted((obj1, obj2) -> obj2.getTotalScore().compareTo(obj1.getTotalScore()))
                            .collect(Collectors.toList())
                            .get(0);
                } else {
                    // 否则：取最后一次的成绩
                    okScore = scores.stream()
                            .sorted((obj1, obj2) -> obj2.getId().compareTo(obj1.getId()))
                            .collect(Collectors.toList())
                            .get(0);
                }

                okExamRecordDataId = okScore.getExamRecordDataId();
                examStudent.setObjectiveScore(okScore.getObjectiveScore());
                examStudent.setSubjectiveScore(okScore.getSubjectiveScore());
                examStudent.setTotalScore(okScore.getTotalScore());
            }

            log.debug("---> {}-{} examStudentId={}, examRecordDataId={}", total, index, examStudent.getExamStudentId(), okExamRecordDataId);

            // 获取试题作答记录
            ExamRecordQuestionVO questions = this.queryExamRecordQuestions(okExamRecordDataId);
            if (questions == null || CollectionUtils.isEmpty(questions.getExamQuestionEntities())) {
                continue;
            }

            // 查询阅卷端考生考试记录中主观题的得分明细
            Map<String, Double> subjectiveQuestionScores = needSubjective ? this.querySubjectiveQuestionScores(okExamRecordDataId) : new HashMap<>();

            List<ExamQuestionVO> examQuestionEntities = questions.getExamQuestionEntities();

            if (!hasDynamicExcelHeaderSetting) {
                for (ExamQuestionVO question : examQuestionEntities) {
                    String questionType = this.convertQuestionType(question.getQuestionType());

                    if ("SINGLE_CHOICE".equals(question.getQuestionType()) || "MULTIPLE_CHOICE".equals(question.getQuestionType()) || "TRUE_OR_FALSE".equals(question.getQuestionType())) {
                        dynamicExcelHeaders.add(String.format("%s%s-%s作答", questionType, question.getMainNumber(), question.getOrder()));
                        defaultDynamicColumnValues.add("");

                        dynamicExcelHeaders.add(String.format("%s%s-%s得分", questionType, question.getMainNumber(), question.getOrder()));
                        defaultDynamicColumnValues.add("0");
                    } else {
                        // 主观题默认不设置“作答”列
                        if (needSubjective) {
                            dynamicExcelHeaders.add(String.format("%s%s-%s作答", questionType, question.getMainNumber(), question.getOrder()));
                            defaultDynamicColumnValues.add("");

                            dynamicExcelHeaders.add(String.format("%s%s-%s得分", questionType, question.getMainNumber(), question.getOrder()));
                            defaultDynamicColumnValues.add("0");
                        }
                    }

                    hasDynamicExcelHeaderSetting = true;

                    paperStructKey += (question.getOrder() + question.getQuestionType());
                }
            }

            String curPaperStructKey = "";// 当前试卷结构关键值
            List<String> curDynamicColumnValues = new ArrayList<>();// Excel动态列当前值
            for (ExamQuestionVO question : examQuestionEntities) {
                if ("SINGLE_CHOICE".equals(question.getQuestionType()) || "MULTIPLE_CHOICE".equals(question.getQuestionType()) || "TRUE_OR_FALSE".equals(question.getQuestionType())) {
                    if ("TRUE_OR_FALSE".equals(question.getQuestionType())) {
                        curDynamicColumnValues.add(this.convertTrueFalse(question.getStudentAnswer()));
                    } else {
                        curDynamicColumnValues.add(this.convertChar(question.getStudentAnswer()));
                    }

                    // 注：客观题得分，目前只能通过比较作答答案是否一致来确定得分
                    String correctAnswer = question.getCorrectAnswer() != null ? question.getCorrectAnswer() : "";
                    String studentAnswer = question.getStudentAnswer() != null ? question.getStudentAnswer() : "";
                    if (correctAnswer.equals(studentAnswer)) {
                        curDynamicColumnValues.add(question.getQuestionScore() != null ? question.getQuestionScore().toString() : "0");
                    } else {
                        curDynamicColumnValues.add("0");
                    }
                } else {
                    // 主观题默认不设置“作答”列
                    if (needSubjective) {
                        curDynamicColumnValues.add(Jsoup.clean(question.getStudentAnswer() != null ? question.getStudentAnswer() : "", Safelist.simpleText()));

                        Double studentScore = subjectiveQuestionScores.get(question.getMainNumber() + "-" + question.getOrder());
                        curDynamicColumnValues.add(studentScore != null ? studentScore.toString() : "0");
                    }
                }

                curPaperStructKey += (question.getOrder() + question.getQuestionType());
            }

            if (!curPaperStructKey.equals(paperStructKey)) {
                String msg = String.format("考生之间存在试卷结构不一致的情况！%s-%s-%s-", examId, courseId, examStudent.getExamStudentId());
                log.warn(msg);
                continue;
                // throw new StatusException("500", msg);
            }

            examStudent.setDetails(curDynamicColumnValues);
        }

        // 补全缺失的作答区
        for (ExamStudentVO examStudent : examStudents) {
            if (CollectionUtils.isNotEmpty(examStudent.getDetails())) {
                continue;
            }
            examStudent.setDetails(defaultDynamicColumnValues);
        }

        List<String> fixedExcelHeaders = Lists.newArrayList(
                // "学习中心", "课程代码", "课程名称", "课程层次", "是否缺考", "身份证号", "学号", "姓名", "年级", "专业", "客观总分", "主观总分", "总分"
                "学习中心", "课程代码", "课程名称", "课程层次", "是否缺考", "身份证号", "学号", "姓名", "客观总分", "主观总分", "总分"
        );
        fixedExcelHeaders.addAll(dynamicExcelHeaders);

        this.toExcel(examId, examName, courseId, examStudents, fixedExcelHeaders);

        examStudents.clear();
    }

    private void toExcel(Long examId, String examName, Long courseId, List<ExamStudentVO> examStudents, List<String> excelHeaders) {
        // 处理Excel
        List<Object[]> excelRows = new ArrayList<>();

        if (CollectionUtils.isNotEmpty(examStudents)) {
            for (ExamStudentVO vo : examStudents) {
                List<Object> rowValues = Lists.newArrayList(
                        vo.getOrgName(),
                        vo.getCourseCode(),
                        vo.getCourseName(),
                        vo.getCourseLevel(),
                        vo.getFinished() ? "否" : "是",
                        vo.getIdentityNumber(),
                        vo.getStudentCode(),
                        vo.getStudentName(),
                        // vo.getGrade(),
                        // vo.getSpecialtyName(),
                        vo.getObjectiveScore().toString(),
                        vo.getSubjectiveScore().toString(),
                        vo.getTotalScore().toString());

                if (CollectionUtils.isNotEmpty(vo.getDetails())) {
                    rowValues.addAll(vo.getDetails());
                }

                excelRows.add(rowValues.toArray(new Object[rowValues.size()]));
            }
        }

        Class<String>[] excelTypes = new Class[excelHeaders.size()];
        for (int n = 0; n < excelHeaders.size(); n++) {
            excelTypes[n] = String.class;
        }

        final String filePath = String.format("/home/admin/project/oe/%s_%s/exam_student_score_%s.xlsx", examId, examName, courseId);
        ExcelWriter.write(excelHeaders.toArray(new String[excelHeaders.size()]), excelTypes, excelRows, new File(filePath));
    }

    private String queryOrgName(Long orgId) {
        final String querySql = String.format("select name from ec_b_org where id = %s", orgId);
        try {
            return jdbcTemplate.queryForObject(querySql, String.class);
        } catch (EmptyResultDataAccessException e) {
            return "";
        }
    }

    private String queryExamName(Long examId) {
        final String querySql = String.format("select name from ec_e_exam where id = %s", examId);
        try {
            return jdbcTemplate.queryForObject(querySql, String.class);
        } catch (EmptyResultDataAccessException e) {
            return "";
        }
    }

    private String queryCourseName(Long courseId) {
        final String querySql = String.format("select name from ec_b_course where id = %s", courseId);
        try {
            return jdbcTemplate.queryForObject(querySql, String.class);
        } catch (EmptyResultDataAccessException e) {
            return "";
        }
    }

    private List<Long> queryExamCourseIds(Long examId) {
        final String querySql = String.format("select distinct course_id from ec_oe_exam_student where exam_id = %s and enable = 1", examId);
        return jdbcTemplate.queryForList(querySql, Long.class);
    }

    private List<ExamStudentVO> queryExamStudents(Long examId, Long courseId) {
        final String columns = "exam_student_id,student_code,student_name,identity_number,org_id,course_id,course_code,course_level,grade,specialty_name,finished";
        final String querySql = String.format("select %s from ec_oe_exam_student where exam_id = %s and course_id = %s and enable = 1", columns, examId, courseId);
        return jdbcTemplate.query(querySql, new BeanPropertyRowMapper(ExamStudentVO.class));
    }

    private String queryExamMarkingType(Long examId, Long markingTypeKey) {
        final String querySql = String.format("select value from ec_e_exam_prop where exam_id = %s and key_id = %s", examId, markingTypeKey);
        try {
            String result = jdbcTemplate.queryForObject(querySql, String.class);
            return StringUtils.isNoneBlank(result) ? result.trim() : "ALL";
        } catch (EmptyResultDataAccessException e) {
            return "ALL";// 默认值
        }
    }

    private List<Map<String, Object>> queryExamRecordDataList(Long examId, Long courseId, Long examStudentId) {
        final String columns = "id,exam_record_status,is_illegality,is_warn,is_audit,base_paper_id";
        final String querySql = String.format("select %s from ec_oe_exam_record_data where exam_id = %s and course_id = %s and exam_student_id = %s", columns, examId, courseId, examStudentId);
        List<Map<String, Object>> result = jdbcTemplate.queryForList(querySql);

        List<Map<String, Object>> okList = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(result)) {
            for (Map<String, Object> row : result) {
                // 过滤考试记录状态情况
                String examRecordStatus = (String) row.get("exam_record_status");
                if (!"EXAM_END".equals(examRecordStatus) && !"EXAM_OVERDUE".equals(examRecordStatus)) {
                    continue;
                }

                Boolean isIllegality = (Boolean) row.get("is_illegality");// 是否违纪
                Boolean isWarn = (Boolean) row.get("is_warn");// 是否警告
                Boolean isAudit = (Boolean) row.get("is_audit");// 是否被审核过

                // 没有违纪的 且 (没有警告 或 有警告已审核通过的)，则有效
                if (!isIllegality && (!isWarn || (isWarn && isAudit))) {
                    okList.add(row);
                }
            }
        }

        return okList;
    }

    private ScoreVO queryExamScore(Long examRecordDataId) {
        final String querySql = String.format("select id,objective_score,subjective_score,total_score from ec_oe_exam_score where exam_record_data_id = %s", examRecordDataId);
        List<ScoreVO> result = jdbcTemplate.query(querySql, new BeanPropertyRowMapper(ScoreVO.class));

        ScoreVO score;
        if (CollectionUtils.isNotEmpty(result)) {
            score = result.get(0);
        } else {
            score = new ScoreVO();
        }

        // 默认值
        score.setExamRecordDataId(examRecordDataId);
        score.setObjectiveScore(score.getObjectiveScore() != null ? score.getObjectiveScore() : 0d);
        score.setSubjectiveScore(score.getSubjectiveScore() != null ? score.getSubjectiveScore() : 0d);
        score.setTotalScore(score.getTotalScore() != null ? score.getTotalScore() : 0d);

        return score;
    }

    private Map<String, Double> querySubjectiveQuestionScores(Long examRecordDataId) {
        StringBuilder querySql = new StringBuilder()
                .append(" select mi.main_number,mi.orders,ri.score")
                .append(" from ec_m_student_paper sp")
                .append(" left join ec_m_mark_item mi on mi.work_id=sp.work_id and mi.base_paper_id=sp.base_paper_id")
                .append(" left join ec_m_mark_result mr on mr.student_paper_id=sp.id")
                .append(" left join ec_m_result_item ri on ri.mark_item_id=mi.id and ri.mark_result_id=mr.id")
                .append(" where sp.exam_record_data_id=").append(examRecordDataId);

        List<Map<String, Object>> result = jdbcTemplate.queryForList(querySql.toString());

        Map<String, Double> scores = new HashMap<>();
        if (CollectionUtils.isNotEmpty(result)) {
            for (Map<String, Object> row : result) {
                Integer mainNumber = (Integer) row.get("main_number");
                Integer orders = (Integer) row.get("orders");
                Double score = (Double) row.get("score");

                scores.put(mainNumber + "-" + orders, score);
            }
        }

        return scores;
    }

    private ExamRecordQuestionVO queryExamRecordQuestions(Long examRecordDataId) {
        Query query = new Query();
        query.addCriteria(Criteria.where("examRecordDataId").is(examRecordDataId));
        List<ExamRecordQuestionVO> result = mongoTemplate.find(query, ExamRecordQuestionVO.class, "examRecordQuestions");
        if (CollectionUtils.isNotEmpty(result)) {
            return result.get(result.size() - 1);
        }
        return null;
    }

    private String convertQuestionType(String type) {
        switch (type) {
            case "SINGLE_CHOICE":
                return "单选题";
            case "MULTIPLE_CHOICE":
                return "多选题";
            case "FILL_UP":
                return "填空题";
            case "TRUE_OR_FALSE":
                return "判断题";
            case "ESSAY":
                return "问答题";
            default:
                return "";
        }
    }

    private String convertChar(String value) {
        if (StringUtils.isBlank(value)) {
            return "";
        }
        return value.replaceAll("0", "A")
                .replaceAll("1", "B")
                .replaceAll("2", "C")
                .replaceAll("3", "D")
                .replaceAll("4", "E")
                .replaceAll("5", "F")
                .replaceAll("6", "G")
                .replaceAll("7", "H")
                .replaceAll("8", "I")
                .replaceAll("9", "J");
    }

    private String convertTrueFalse(String value) {
        if ("true".equals(value)) {
            return "对";
        } else if ("false".equals(value)) {
            return "错";
        } else if ("1".equals(value)) {
            return "对";
        } else if ("0".equals(value)) {
            return "错";
        } else {
            return "";
        }
    }

}