package cn.com.qmth.dp.examcloud.oe.modules.fixExamStudentId;

import cn.com.qmth.dp.examcloud.oe.modules.fixExamStudentId.vo.ExamStudentVO;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 修复考务考生表某些考生被不合理删除错误（可重复执行）
 */
@Component
public class FixExamStudentId {

    private final static Logger log = LoggerFactory.getLogger(FixExamStudentId.class);

    @Autowired
    private JdbcTemplate jdbcTemplate;

    private static int errRecord = 0;

    public void start(Long examId, Long courseId) {
        // 查询网考端考生表记录
        StringBuilder querySql = new StringBuilder()
                .append("select id,exam_id,course_id,student_id,exam_student_id ")
                .append(" from ec_oe_exam_student ")
                .append(" where id > %s and exam_id = %s ")
                .append(courseId != null ? " and course_id = %s " : "%s")
                .append(" order by id asc limit 0,1000");

        int total = 0;
        Long nextMinId = 0L;
        while (true) {
            System.out.println();
            log.info("query nextMinId:" + nextMinId + ", total " + total + ", errRecord " + errRecord);

            List<ExamStudentVO> list = jdbcTemplate.query(
                    String.format(querySql.toString(), nextMinId, examId, courseId != null ? courseId : ""),
                    new BeanPropertyRowMapper(ExamStudentVO.class));

            if (CollectionUtils.isEmpty(list)) {
                break;
            }

            nextMinId = list.get(list.size() - 1).getId();
            total += list.size();

            this.fix(list);
        }

        log.info("task end...");
    }

    private void fix(List<ExamStudentVO> examStudentList) {
        for (ExamStudentVO vo : examStudentList) {
            // 查询考务端考生表记录
            Long realExamStudentId = this.queryExamStudentFromExamWork(vo.getExamId(), vo.getCourseId(), vo.getStudentId());
            if (realExamStudentId == null) {
                log.warn("curExamStudentId = " + vo.getExamStudentId() + " realExamStudentId is not exist");
                errRecord++;
                continue;
            }

            if (!vo.getExamStudentId().equals(realExamStudentId)) {
                StringBuilder msg = new StringBuilder();
                msg.append(" curExamStudentId = ").append(vo.getExamStudentId());
                msg.append(" realExamStudentId = ").append(realExamStudentId);

                int a;
                if (this.exist_ec_oe_exam_student_final_score(realExamStudentId)) {
                    a = this.delete_ec_oe_exam_student_final_score(vo.getExamStudentId());
                    msg.append("\n delete_ec_oe_exam_student_final_score - ").append(a);
                } else {
                    a = this.update_ec_oe_exam_student_final_score(vo.getExamStudentId(), realExamStudentId);
                    msg.append("\n update_ec_oe_exam_student_final_score - ").append(a);
                }

                int b = this.update_ec_oe_exam_record_4_marking(vo.getExamStudentId(), realExamStudentId);
                msg.append("\n update_ec_oe_exam_record_4_marking - ").append(b);

                int c = this.update_ec_oe_exam_record_data(vo.getExamStudentId(), realExamStudentId);
                msg.append("\n update_ec_oe_exam_record_data - ").append(c);

                int d = this.update_ec_oes_exam_record_data(vo.getExamStudentId(), realExamStudentId);
                msg.append("\n update_ec_oes_exam_record_data - ").append(d);

                int e = this.delete_ec_oe_exam_student(vo.getId());
                msg.append("\n delete_ec_oe_exam_student - ").append(e);

                errRecord++;

                log.info(msg.toString());
            }
        }
    }

    private Long queryExamStudentFromExamWork(Long examId, Long courseId, Long studentId) {
        String querySql = String.format("select id from ec_e_exam_student where exam_id = %s and course_id = %s and student_id = %s ", examId, courseId, studentId);

        try {
            return jdbcTemplate.queryForObject(querySql, Long.class);
        } catch (EmptyResultDataAccessException e) {
            return null;
        }
    }

    private boolean exist_ec_oe_exam_student_final_score(Long curExamStudentId) {
        String countSql = String.format("select count(1) from ec_oe_exam_student_final_score where exam_student_id = %s", curExamStudentId);
        Long total = jdbcTemplate.queryForObject(countSql, Long.class);
        return total > 0 ? true : false;
    }

    private int delete_ec_oe_exam_student_final_score(Long curExamStudentId) {
        String sql = String.format("delete from ec_oe_exam_student_final_score where exam_student_id = %s", curExamStudentId);
        return jdbcTemplate.update(sql);
    }

    private int update_ec_oe_exam_student_final_score(Long curExamStudentId, Long realExamStudentId) {
        String sql = String.format("update ec_oe_exam_student_final_score set exam_student_id = %s where exam_student_id = %s", realExamStudentId, curExamStudentId);
        return jdbcTemplate.update(sql);
    }

    private int update_ec_oe_exam_record_4_marking(Long curExamStudentId, Long realExamStudentId) {
        String sql = String.format("update ec_oe_exam_record_4_marking set exam_student_id = %s where exam_student_id = %s", realExamStudentId, curExamStudentId);
        return jdbcTemplate.update(sql);
    }

    private int update_ec_oe_exam_record_data(Long curExamStudentId, Long realExamStudentId) {
        String sql = String.format("update ec_oe_exam_record_data set exam_student_id = %s where exam_student_id = %s", realExamStudentId, curExamStudentId);
        return jdbcTemplate.update(sql);
    }

    private int update_ec_oes_exam_record_data(Long curExamStudentId, Long realExamStudentId) {
        String sql = String.format("update ec_oes_exam_record_data set exam_student_id = %s where exam_student_id = %s", realExamStudentId, curExamStudentId);
        return jdbcTemplate.update(sql);
    }

    private int delete_ec_oe_exam_student(Long id) {
        String sql = String.format("delete from ec_oe_exam_student where id = %s", id);
        return jdbcTemplate.update(sql);
    }

}
