package cn.com.qmth.dp.examcloud.oe.modules.get_student_answer_by_identity_number;

import java.io.File;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.bson.Document;
import org.jsoup.Jsoup;
import org.jsoup.safety.Whitelist;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.data.mongodb.core.DocumentCallbackHandler;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;
import com.mongodb.MongoException;

import cn.com.qmth.examcloud.commons.helpers.ObjectHolder;
import cn.com.qmth.examcloud.commons.helpers.poi.ExcelWriter;
import cn.com.qmth.examcloud.commons.util.ResourceLoader;

/**
 * 获取一个考生的作答明细
 *
 * @author WANGWEI
 * @date 2019年7月11日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class GetStduentAnswerByIdentityNumberService {

	@Autowired
	JdbcTemplate jdbcTemplate;

	@Autowired
	MongoTemplate mongoTemplate;

	private int sqlIndex = 1;

	// @Async
	public void start(Long examId, String courseCode, String identityNumber) throws Exception {
		String packageName = this.getClass().getPackage().getName();
		String packagePath = packageName.replaceAll("\\.", "/");
		String sql = ResourceLoader
				.getResource(packagePath + "/query_exam_record_data_" + sqlIndex + ".sql");
		Object[] args = new Object[]{examId, courseCode, identityNumber};
		List<Map<String, Object>> result = jdbcTemplate.queryForList(sql, args);

		List<Long> examRecordDataIdList = Lists.newArrayList();

		for (Map<String, Object> map : result) {
			Long examRecordDataId = (Long) map.get("EXAM_RECORD_DATA_ID");
			String curIdentityNumber = (String) map.get("IDENTITY_NUMBER");
			String curCourseCode = (String) map.get("CODE");

			if (!curCourseCode.equals(courseCode)) {
				throw new RuntimeException("unexpected exception. courseCode=" + courseCode);
			}

			if (!curIdentityNumber.equals(identityNumber)) {
				throw new RuntimeException("unexpected exception. courseCode=" + courseCode);
			}

			examRecordDataIdList.add(examRecordDataId);
		}

		List<Object[]> datas = Lists.newArrayList();

		for (Long cur : examRecordDataIdList) {

			List<Document> answers = getAnswers(cur);

			for (Document doc : answers) {
				Integer order = doc.getInteger("order");
				Double studentScore = doc.getDouble("studentScore");
				String questionId = doc.getString("questionId");
				String studentAnswer = doc.getString("studentAnswer");
				String questionType = doc.getString("questionType");

				if (null == order) {
					throw new RuntimeException("order is null. examRecordDataId=" + cur);
				}

				if (null == questionType) {
					throw new RuntimeException(
							"questionType is null. examRecordDataId=" + cur + "; order=" + order);
				}

				if (null == studentScore) {
					// 无作答时,0分
					if (StringUtils.isBlank(studentAnswer)) {
						studentScore = 0D;
					}
				}

				if (null != studentAnswer && !studentAnswer.contains("<img")) {
					studentAnswer = Jsoup.clean(studentAnswer, Whitelist.simpleText());
				}

				if (null != studentAnswer) {
					if (questionType.equals("SINGLE_CHOICE")
							|| questionType.equals("MULTIPLE_CHOICE")) {
						studentAnswer = studentAnswer.replaceAll("0", "A");
						studentAnswer = studentAnswer.replaceAll("1", "B");
						studentAnswer = studentAnswer.replaceAll("2", "C");
						studentAnswer = studentAnswer.replaceAll("3", "D");
						studentAnswer = studentAnswer.replaceAll("4", "E");
						studentAnswer = studentAnswer.replaceAll("5", "F");
						studentAnswer = studentAnswer.replaceAll("6", "G");
						studentAnswer = studentAnswer.replaceAll("7", "H");
						studentAnswer = studentAnswer.replaceAll("8", "I");
					}
				}

				Object[] row = new Object[]{identityNumber, courseCode, String.valueOf(cur),
						String.valueOf(order), questionId, studentAnswer,
						null != studentScore ? String.valueOf(studentScore) : null};

				datas.add(row);
			}

		}

		String filePath = "D:/Temp/answers-" + examId + "-" + courseCode + "-" + identityNumber
				+ ".xlsx";

		final String[] EXCEL_HEADER = new String[]{"身份证号码", "课程代码", "考试记录ID", "题号", "题目ID", "答案",
				"分数"};

		ExcelWriter
				.write(EXCEL_HEADER,
						new Class[]{String.class, String.class, String.class, String.class,
								String.class, String.class, String.class},
						datas, new File(filePath));

		System.out.println("OVER !   identityNumber=" + identityNumber);
	}

	/**
	 * 获取作答
	 *
	 * @author WANGWEI
	 * @param examRecordDataId
	 * @return
	 */
	private List<Document> getAnswers(Long examRecordDataId) {
		Query query = Query.query(Criteria.where("examRecordDataId").is(examRecordDataId));
		final ObjectHolder<List<Document>> answersHolder = new ObjectHolder<List<Document>>(null);
		mongoTemplate.executeQuery(query, "examRecordQuestions", new DocumentCallbackHandler() {

			@SuppressWarnings("unchecked")
			@Override
			public void processDocument(Document document)
					throws MongoException, DataAccessException {
				List<Document> answers = (List<Document>) document.get("examQuestionEntities");
				answersHolder.set(answers);
			}
		});

		List<Document> answers = answersHolder.get();
		return answers;
	}

	public void setSqlIndex(int sqlIndex) {
		this.sqlIndex = sqlIndex;
	}

}
