package cn.com.qmth.dp.examcloud.oe.modules.import_data;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import cn.com.qmth.dp.examcloud.oe.modules.export_data.ExportData;
import cn.com.qmth.dp.examcloud.oe.util.DBUtil;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import cn.com.qmth.examcloud.commons.helpers.Counter;
import cn.com.qmth.examcloud.commons.util.DateUtil;
import cn.com.qmth.examcloud.commons.util.DateUtil.DatePatterns;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.commons.util.ResourceLoader;
import cn.com.qmth.examcloud.commons.util.Util;

@Component
public class ImportData {

	private static final Logger log = LoggerFactory.getLogger(ImportData.class);

	public void start(String dirPath) {

		new Thread(new Runnable() {
			@Override
			public void run() {
				Counter counter = new Counter(0, Long.MAX_VALUE);
				while (true) {
					File dir = new File(dirPath);

					File[] fileArr = dir.listFiles();

					for (File file : fileArr) {
						processFile(counter, file);
					}

					Util.sleep(60);
				}

			}
		}).start();
	}

	private void processFile(Counter counter, File file) {

		String packageName = this.getClass().getPackage().getName();
		String packagePath = packageName.replaceAll("\\.", "/");
		String insertSql = ResourceLoader.getResource(packagePath + "/insert_data.sql");
		String deleteSql = ResourceLoader.getResource(packagePath + "/delete_data.sql");

		BufferedReader br = null;
		InputStreamReader reader = null;

		boolean hasError = false;
		try {
			reader = new InputStreamReader(new FileInputStream(file), "UTF-8");
			br = new BufferedReader(reader);

			Connection ds2Conn = DBUtil.getConnection("ds2");

			String line = null;
			while (null != (line = br.readLine())) {
				Object[] deleteParams = null;
				Object[] columValues = null;
				try {
					columValues = toColumValues(line);
					deleteParams = getDeleteParams(columValues);
				} catch (Exception e) {
					log.error("[DataImport]. ErrorLine: " + line, e);
					continue;
				}

				try {
					delete(ds2Conn, deleteSql, deleteParams);
				} catch (Exception e) {
					hasError = true;
					log.error("[DataImport]. DeleteError: " + line, e);
				}

				try {
					insert(ds2Conn, insertSql, columValues);
					counter.next();
					log.info("[DataImport]. count=" + counter.get());
				} catch (Exception e) {
					hasError = true;
					log.error("[DataImport]. InsertError: " + line, e);
				}

			}

		} catch (Exception e) {
			hasError = true;
			log.error("[DataImport]. unexpected", e);
		} finally {
			IOUtils.closeQuietly(br);
			IOUtils.closeQuietly(reader);
		}

		if (!hasError) {
			try {
				FileUtils.forceDelete(file);
			} catch (IOException e) {
				log.error("[DataImport]. fail to delete file. path="
						+ PathUtil.getCanonicalPath(file), e);
			}
		}

	}

	private void insert(Connection ds2Conn, String insertSql, Object[] columValues)
			throws SQLException {

		while (!ds2Conn.isValid(1)) {
			ds2Conn = DBUtil.getConnection("ds2");
		}

		PreparedStatement ps2 = null;
		try {
			ps2 = ds2Conn.prepareStatement(insertSql);
			for (int i = 0; i < columValues.length; i++) {
				ps2.setObject(i + 1, columValues[i]);
			}
			ps2.execute();
		} finally {
			DBUtil.close(ps2);
		}

	}

	private void delete(Connection ds2Conn, String deleteSql, Object[] params) throws SQLException {

		while (!ds2Conn.isValid(1)) {
			ds2Conn = DBUtil.getConnection("ds2");
		}

		PreparedStatement ps2 = null;
		try {
			ps2 = ds2Conn.prepareStatement(deleteSql);
			for (int i = 0; i < params.length; i++) {
				ps2.setObject(i + 1, params[i]);
			}
			ps2.execute();
		} finally {
			DBUtil.close(ps2);
		}

	}

	private Object[] toColumValues(String str) {
		JsonObject jsonObj = JsonParser.parseString(str).getAsJsonObject();

		Object[] arr = new Object[9];

		arr[0] = jsonObj.get("id").getAsLong();
		arr[1] = jsonObj.get("root_org_id").getAsLong();
		arr[2] = jsonObj.get("exam_student_id").getAsLong();
		arr[3] = jsonObj.get("exam_record_data_id").getAsLong();
		arr[4] = jsonObj.get("photo_path").getAsString();
		arr[5] = jsonObj.get("file_url").getAsString();
		arr[6] = jsonObj.get("face_compare_result").getAsString();
		arr[7] = DateUtil.parse(jsonObj.get("creation_time").getAsString(),
				DatePatterns.CHINA_DEFAULT);
		if (!jsonObj.get("used_time").isJsonNull()) {
			arr[8] = jsonObj.get("used_time").getAsLong();
		}

		return arr;
	}

	private Object[] getDeleteParams(Object[] columValues) {
		Object[] arr = new Object[1];
		arr[0] = columValues[0];
		return arr;
	}

}
