package cn.com.qmth.dp.examcloud.oe.modules.marking_item_change;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.transaction.Transactional;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;

import cn.com.qmth.dp.examcloud.oe.bean.MarkItemBean;
import cn.com.qmth.dp.examcloud.oe.bean.MarkItemEntity;
import cn.com.qmth.dp.examcloud.oe.bean.MarkRangeEntity;
import cn.com.qmth.dp.examcloud.oe.bean.MarkWorkEntity;
import cn.com.qmth.dp.examcloud.oe.modules.update_correct_answer.util.OKHttpUtil;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.ByteUtil;
import cn.com.qmth.examcloud.commons.util.SHA256;
import cn.com.qmth.examcloud.commons.util.StringUtil;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import okhttp3.FormBody;
import okhttp3.MediaType;
import okhttp3.Request;
import okhttp3.RequestBody;

/**
 * 修改评卷任务给分板
 * 
 * @author chenken
 *
 */
@Service
public class MarkingItemChangeService {
	private String requestUrl = "http://192.168.1.227:8008/api/core/questions/extract_config/getBasePaper";
	private long appId = 3;
	private String appCode = "Q";
	private String secretKey = "123456";

	@Autowired
	private JdbcTemplate jdbcTemplate;

	@Transactional
	public void start() {
		System.out.println("*******************start");
		List<String> basePaperIds = new ArrayList<>();
		basePaperIds.add("60c18496c68d4c45f68ef951");
		basePaperIds.add("60c18495c68d4c45f68ef7af");
		
		Long workId = 1479L;

//		basePaperIds.add("60c9b131900856162564c758");
//		basePaperIds.add("60b737ad9008565a0e20f785");
//
//		Long workId = 139L;

		for (String basePaperId : basePaperIds) {
			List<MarkItemBean> markItems = new ArrayList<MarkItemBean>();
			DefaultPaper defaultPaper = getBaseDefaultPaper(basePaperId);
			List<DefaultQuestionGroup> defaultQuestionGroupList = defaultPaper.getQuestionGroupList();
			// 大题号
			int mainNumber = 1;
			// 每个小题在试卷中的位置
			int order = 0;
			for (DefaultQuestionGroup defaultQuestionGroup : defaultQuestionGroupList) {
				List<DefaultQuestionStructureWrapper> defaultQuestionStructureWrapperList = defaultQuestionGroup
						.getQuestionWrapperList();
				// 小题号
				int subNumber = 1;
				for (DefaultQuestionStructureWrapper defaultQuestionStructureWrapper : defaultQuestionStructureWrapperList) {
					List<DefaultQuestionUnitWrapper> defaultQuestionUnitWrapperList = defaultQuestionStructureWrapper
							.getQuestionUnitWrapperList();
					for (DefaultQuestionUnitWrapper defaultQuestionUnitWrapper : defaultQuestionUnitWrapperList) {
						order++;
						if ("FILL_UP".equals(defaultQuestionUnitWrapper.getQuestionType().toString())
								|| "ESSAY".equals(defaultQuestionUnitWrapper.getQuestionType().toString())) {
							MarkItemBean markItem = new MarkItemBean(basePaperId, mainNumber, subNumber,
									defaultQuestionUnitWrapper.getQuestionScore(), 0.5, order,
									defaultQuestionStructureWrapper.getQuestionId());
							markItems.add(markItem);
						}
						subNumber++;
					}
				}
				mainNumber++;
			}
			saveMarkWorkItem(workId, markItems, basePaperId);
		}
		System.out.println("*******************finish");
	}

	private void saveMarkWorkItem(Long workId, List<MarkItemBean> markItemBeanList, String basePaperId) {
		if (null != workId && CollectionUtils.isNotEmpty(markItemBeanList)) {
			List<MarkRangeEntity> rs = findByWorkIdAndBasePaperId(workId, basePaperId);
			MarkWorkEntity work = findByWorkId(workId);
			if (work == null) {
				throw new StatusException("Q-020560", "work null");
			}
			List<MarkItemEntity> markItems = Lists.newArrayList();
			for (MarkItemBean bean : markItemBeanList) {
				MarkItemEntity entity = new MarkItemEntity(bean.getBasePaperId(), bean.getMainNumber(),
						bean.getSubNumber(), bean.getMaxScore(), bean.getScoreInterval(), bean.getOrders(),
						bean.getQuestionId());
				entity.setWorkId(workId);
				entity.setMarkRangeId(rs.get(0).getId());
				markItems.add(entity);
			}
			deleteByWorkIdPaperId(workId, basePaperId);
			for (MarkItemEntity markItem : markItems) {
				saveMarkItem(markItem);
			}
		}
	}

	private void saveMarkItem(MarkItemEntity markItem) {
		String sql = "INSERT INTO `ec_m_mark_item` "
				+ "(`base_paper_id`, `main_number`, `mark_range_id`, `max_score`, `orders`, `question_id`, `score_interval`, `scoring_type`, `socre_list`, `sub_number`, `work_id`) "
				+ "VALUES ('" + markItem.getBasePaperId() + "', " + markItem.getMainNumber() + ", "
				+ markItem.getMarkRangeId() + ", " + markItem.getMaxScore() + ", " + markItem.getOrders() + ", '"
				+ markItem.getQuestionId() + "', " + markItem.getScoreInterval() + ", '" + markItem.getScoringType()
				+ "', NULL, " + markItem.getSubNumber() + ", " + markItem.getWorkId() + ");";
		jdbcTemplate.update(sql);
	}

	private void deleteByWorkIdPaperId(Long workId, String basePaperId) {
		StringBuilder sqlBuilder = new StringBuilder();
		sqlBuilder.append(" delete from ec_m_mark_item  ");
		sqlBuilder.append(" where work_id =" + workId);
		sqlBuilder.append(" and base_paper_id ='" + basePaperId + "'");
		jdbcTemplate.update(sqlBuilder.toString());
	}

	private List<MarkRangeEntity> findByWorkIdAndBasePaperId(Long workId, String basePaperId) {
		StringBuilder sqlBuilder = new StringBuilder();
		sqlBuilder.append(" select t.* from ec_m_mark_range t ");
		sqlBuilder.append(" where t.work_id =" + workId);
		sqlBuilder.append(" and t.base_paper_id ='" + basePaperId + "'");

		RowMapper<MarkRangeEntity> rowMapper = new BeanPropertyRowMapper<MarkRangeEntity>(MarkRangeEntity.class);
		List<MarkRangeEntity> ret = jdbcTemplate.query(sqlBuilder.toString(), rowMapper);
		return ret;
	}

	private MarkWorkEntity findByWorkId(Long workId) {
		StringBuilder sqlBuilder = new StringBuilder();
		sqlBuilder.append(" select t.* from ec_m_mark_work t ");
		sqlBuilder.append(" where t.id =" + workId);

		RowMapper<MarkWorkEntity> rowMapper = new BeanPropertyRowMapper<MarkWorkEntity>(MarkWorkEntity.class);
		MarkWorkEntity ret = jdbcTemplate.queryForObject(sqlBuilder.toString(), rowMapper);
		return ret;
	}

	private DefaultPaper getBaseDefaultPaper(String paperId) {

		long timestamp = System.currentTimeMillis();
		String joinStr = StringUtil.join(appId, appCode, timestamp, secretKey);
		byte[] bytes = SHA256.encode(joinStr);
		String accessToken = ByteUtil.toHexAscii(bytes);
		Map<String, String> params = new HashMap<>();
		params.put("paperId", paperId);
		try {
			String res = rpcPost(appId, appCode, accessToken, timestamp, requestUrl, params);
			JSONObject jo = JSONObject.parseObject(res);
			DefaultPaper d = JSONObject.parseObject(jo.getString("defaultPaper"), DefaultPaper.class);
			return d;
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private String rpcPost(Long appId, String appCode, String accessToken, long timestamp, String requestUrl,
			Map<String, String> params) throws IOException {
		final String contentType = "application/json; charset=utf-8";
		RequestBody formBody = FormBody.create(MediaType.parse(contentType), JSONObject.toJSONString(params));
		Request.Builder request = new Request.Builder().url(requestUrl).header("App-Id", String.valueOf(appId))
				.header("App-Code", appCode).header("Access-Token", accessToken)
				.header("timestamp", String.valueOf(timestamp)).post(formBody);

		String resp = OKHttpUtil.call(request);
		return resp;
	}
}
