package cn.com.qmth.dp.examcloud.oe.modules.update_student_answer;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.RegExUtils;
import org.assertj.core.util.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import com.google.common.collect.Lists;

import cn.com.qmth.dp.examcloud.oe.modules.update_student_answer.entity.FixAnswerType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.helpers.Counter;
import cn.com.qmth.examcloud.commons.util.JsonUtil;
import cn.com.qmth.examcloud.commons.util.Util;
import cn.com.qmth.examcloud.web.upyun.UpyunService;

/**
 * @Description 修复学生作答
 * @Author THINKPAD
 * @Date 2019/7/8 10:16
 * @Version 1.0
 */
@Service
public class FixStudentAnswerService {

	@Autowired
	JdbcTemplate jdbcTemplate;

	@Autowired
	MongoTemplate mongoTemplate;

	@Autowired
	UpyunService upyunService;

	private static Counter counter = new Counter(0L, 1000000L);

	public void fixStudentAnswer(String filePath, Long examId, String courseCode, Integer order,
			FixAnswerType fixAnswerType)
			throws FileNotFoundException, UnsupportedEncodingException {
		if (fixAnswerType != FixAnswerType.AUDIO && fixAnswerType != FixAnswerType.PICTURE) {
			throw new StatusException("100001", "修改的文件类型格式不正确");
		}
		// 错误提示
		List<Map<String, Object>> failList = Lists.newArrayList();
		File file = new File(filePath);

		if (file.isDirectory()) {
			String[] list = file.list();
			// 循环查库
			// 获取身份证键值对信息
			Map<String, List<Map<String, Object>>> identityMap = getIdentityMap(filePath, list);

			Set<String> keys = identityMap.keySet();
			Set<String> tempKeys = Sets.newHashSet();
			int index = 0;
			int threadSize = 10;
			int batchSize = keys.size() / threadSize + 1;
			for (String key : keys) {
				index++;
				tempKeys.add(key);
				if (index == batchSize) {
					new FixStudentAnswerThread(jdbcTemplate, mongoTemplate, tempKeys, examId,
							courseCode, failList, identityMap, order, fixAnswerType, upyunService,
							counter).start();
					index = 0;
					tempKeys = Sets.newHashSet();
				}
			}

			if (CollectionUtils.isNotEmpty(tempKeys)) {
				new FixStudentAnswerThread(jdbcTemplate, mongoTemplate, tempKeys, examId,
						courseCode, failList, identityMap, order, fixAnswerType, upyunService,
						counter).start();
			}

		}

		while (true) {
			Util.sleep(5);
			System.out.println("count: " + counter.get());
			System.out.println("failList: \n" + JsonUtil.toPrettyJson(failList));
		}

	}

	/**
	 * 获取身份证和文件地址的对应关系
	 * 
	 * @param list
	 * @return
	 */
	private Map<String, List<Map<String, Object>>> getIdentityMap(String filePath, String[] list) {
		// 所有的图片
		List<Map<String, Object>> fileMapList = new ArrayList<>();
		for (String fileName : list) {
			String fullFilePath = filePath + File.separator + fileName;
			fileName = RegExUtils.replaceAll(fileName, "\\s+", "");
			// 如果有-，则证明有多张图片
			if (fileName.indexOf("-") != -1) {
				String identityNumber = fileName.substring(0, fileName.indexOf("-"));
				Map<String, Object> map = new HashMap<>();
				map.put("identityNumber", identityNumber);
				map.put("filePath", fullFilePath);
				// 排序号
				int sortOrder = 0;
				if (fileName.indexOf("_") != -1) {
					try {
						sortOrder = Integer.parseInt(fileName.substring(fileName.indexOf("-") + 1,
								fileName.indexOf("_")));
					} catch (NumberFormatException e) {
						e.printStackTrace();
					}
				} else {
					try {
						sortOrder = Integer.parseInt(fileName.substring(fileName.indexOf("-") + 1,
								fileName.indexOf(".")));
					} catch (NumberFormatException e) {
						e.printStackTrace();
					}
				}
				map.put("order", sortOrder);
				fileMapList.add(map);
			} else {
				String identityNumber = "";
				if (fileName.indexOf("_") > -1) {
					identityNumber = fileName.substring(0, fileName.indexOf("_"));
				} else {
					identityNumber = fileName.substring(0, fileName.indexOf("."));
				}
				Map<String, Object> map = new HashMap<>();
				map.put("identityNumber", identityNumber);
				map.put("filePath", fullFilePath);
				map.put("order", 0);
				fileMapList.add(map);
			}
		}
		Map<String, List<Map<String, Object>>> resultMap = new HashMap<>();
		for (Map<String, Object> fm : fileMapList) {
			List<Map<String, Object>> mapList = fileMapList.stream()
					.filter(p -> p.get("identityNumber").equals(fm.get("identityNumber")))
					.collect(Collectors.toList());
			// 对文件进行排序
			Collections.sort(mapList, new Comparator<Map<String, Object>>() {
				public int compare(Map<String, Object> o1, Map<String, Object> o2) {
					int map1value = Integer.parseInt(o1.get("order").toString());
					int map2value = Integer.parseInt(o2.get("order").toString());
					return map1value - map2value;
				}
			});
			if (resultMap.get(fm.get("identityNumber")) == null) {
				resultMap.put(fm.get("identityNumber").toString(), mapList);
			}
		}
		return resultMap;
	}
}
